@@ -67,19 +67,21 @@
 #include "../multiif.h"
 #include "../curlx/fopen.h"
 #include "../curlx/timeval.h"
+#include "../curl_md5.h"
 #include "../curl_sha256.h"
 #include "../curlx/warnless.h"
 #include "../curlx/base64.h"
+#include "../curl_printf.h"
 #include "../curlx/inet_pton.h"
 #include "../connect.h"
 #include "../select.h"
 #include "../setopt.h"
-#include "../rand.h"
 #include "../strdup.h"
+#include "../rand.h"
 
 #ifdef USE_APPLE_SECTRUST
 #include <Security/Security.h>
-#endif
+#endif /* USE_APPLE_SECTRUST */
 
 /* The last #include files should be: */
 #include "../curl_memory.h"
@@ -1092,8 +1094,8 @@ static size_t multissl_version(char *buffer, size_t size)
       bool paren = (selected != available_backends[i]);
 
       if(available_backends[i]->version(vb, sizeof(vb))) {
-        p += curl_msnprintf(p, end - p, "%s%s%s%s", (p != backends ? " " : ""),
-                            (paren ? "(" : ""), vb, (paren ? ")" : ""));
+        p += msnprintf(p, end - p, "%s%s%s%s", (p != backends ? " " : ""),
+                       (paren ? "(" : ""), vb, (paren ? ")" : ""));
       }
     }
 
@@ -1364,15 +1366,6 @@ static CURLcode ssl_cf_connect(struct Curl_cfilter *cf,
   DEBUGASSERT(connssl);
 
   *done = FALSE;
-
-  if(!connssl->prefs_checked) {
-    if(!ssl_prefs_check(data)) {
-      result = CURLE_SSL_CONNECT_ERROR;
-      goto out;
-    }
-    connssl->prefs_checked = TRUE;
-  }
-
   if(!connssl->peer.hostname) {
     char tls_id[80];
     connssl->ssl_impl->version(tls_id, sizeof(tls_id) - 1);
@@ -1381,6 +1374,12 @@ static CURLcode ssl_cf_connect(struct Curl_cfilter *cf,
       goto out;
   }
 
+  if(!connssl->prefs_checked) {
+    if(!ssl_prefs_check(data))
+      return CURLE_SSL_CONNECT_ERROR;
+    connssl->prefs_checked = TRUE;
+  }
+
   result = connssl->ssl_impl->do_connect(cf, data, done);
 
   if(!result && *done) {
@@ -1858,8 +1857,7 @@ static CURLcode vtls_shutdown_blocking(struct Curl_cfilter *cf,
     if(timeout_ms < 0) {
       /* no need to continue if time is already up */
       failf(data, "SSL shutdown timeout");
-      result = CURLE_OPERATION_TIMEDOUT;
-      goto out;
+      return CURLE_OPERATION_TIMEDOUT;
     }
 
     result = connssl->ssl_impl->shut_down(cf, data, send_shutdown, done);
@@ -2062,9 +2060,11 @@ CURLcode Curl_alpn_set_negotiated(struct Curl_cfilter *cf,
       result = CURLE_SSL_CONNECT_ERROR;
       goto out;
     }
-    connssl->negotiated.alpn = Curl_memdup0((const char *)proto, proto_len);
+    connssl->negotiated.alpn = malloc(proto_len + 1);
     if(!connssl->negotiated.alpn)
       return CURLE_OUT_OF_MEMORY;
+    memcpy(connssl->negotiated.alpn, proto, proto_len);
+    connssl->negotiated.alpn[proto_len] = 0;
   }
 
   if(proto && proto_len) {
